#
# TclTk windowing routines for cnet
#
# Chris McDonald and Michael J. Robins, Feb->Apr 95.
#
#
proc MkMainWindow {w} {
    set CNET_ICON		/home/bison/staff/chris/include/icon.xbm
    wm withdraw .
    toplevel $w
    wm minsize $w 0 0
    wm iconbitmap $w "@$CNET_ICON"
}

proc MkTitleBar {w progname deldstr exitproc} {
    global CN_DELIVERY

    frame $w
    button $w.exit	-text	"Exit $progname" \
	-command $exitproc \
	-relief raised -bd 2
    label $w.msgs	-text	"Messages delivered correctly:"
    set CN_DELIVERY	"$deldstr"
    label $w.deld	-textvariable CN_DELIVERY

    pack $w.exit $w.msgs $w.deld -side left -fill x -expand yes
    pack $w -side top -padx 4 -pady 2 -fill x -expand no
}

proc MkFrameTop {w runproc saveproc} {
    set f [frame $w.top]
    button $f.run	-textvariable CN_RUN_STRING \
	-command	"$runproc run" \
	-width 8 \
	-relief raised -bd 2
    button $f.single	-text	"Single event" \
	-command	"$runproc step" \
	-relief raised -bd 2
    button $f.save	-text	"Save topology" \
	-command	"ToggleSave $saveproc" \
	-relief raised -bd 2
    button $f.histo	-text	"Histograms" \
	-command { ToggleHistogram } \
	-relief raised -bd 2

    pack $f.run $f.single $f.save $f.histo -side left -fill x -expand yes
    pack $f -side top -padx 4 -pady 2 -fill x -expand no
}

proc MkFrameBot {w} {
    set f [frame $w.bot]
    button $f.nodes	-text	"Default node attributes"	\
	-relief raised -bd 2 \
	-command "ToggleNodeAttributes -1 default default $w 0 0 0 0 0"
    button $f.links	-text	"Default link attributes"	\
	-relief raised -bd 2 \
	-command "ToggleLinkAttributes -1 -1 default $w 0 0 0 0"
    button $f.stats	-text	"Statistics"   \
	-relief raised -bd 2 \
	-command { ToggleStatistics }

    pack $f.nodes $f.links $f.stats -side left -fill x -expand yes
    pack $f -side top -padx 4 -pady 2 -fill x -expand no
}

proc MkCanvas {w width height} {
    if {[info command $w] == ""} {
	canvas $w -width $width -height $height  \
	    -background white -relief sunken -border 2
	pack $w -side top -fill both -expand yes
     }
}

# ----------------------------------------------------------------

proc ToggleHistogram {} {
    global CN_HISTO_WIDTH CN_HISTO_HEIGHT histogram_displayed

    set w .hist
    if {[info command $w] == ""} {
	toplevel $w
	MkCanvasFrame $w.top "histograms" "ToggleHistogram"
	MkCanvas $w.can $CN_HISTO_WIDTH $CN_HISTO_HEIGHT
	set histogram_displayed true
	histogram_created $w.can
    } elseif {[wm state $w] == "withdrawn"} {
	wm deiconify $w
	set histogram_displayed true
    } else {
	wm withdraw $w
	set histogram_displayed false
    }
}

proc SaveTopology {saveproc e} {
    $saveproc [$e get]
    wm withdraw .save
}

proc ToggleSave {saveproc} {
    set w .save
    if {[info command $w] == ""} {
	toplevel $w
	set s [frame $w.f]
	label $s.l	-text "Topology filename:"
	entry $s.e	-background white -width 30
	bind $s.e <Return> "SaveTopology $saveproc $s.e"
	pack $s.l $s.e -side left -fill both -expand yes
	pack $s

	button $w.save	 -text save	-command "SaveTopology $saveproc $s.e"
	button $w.cancel -text cancel	-command "wm withdraw $w"
	pack $w.save $w.cancel -side left -fill both -expand yes
    } elseif {[wm state $w] == "withdrawn"} {
	wm deiconify $w
    } else {
	wm withdraw $w
    }
}

proc ToggleStatistics {} {
    global CN_STATS_WIDTH CN_STATS_HEIGHT stats_displayed

    set w .stats
    if {[info command $w] == ""} {
	toplevel $w
	MkCanvasFrame $w.top "statistics" "ToggleStatistics"
	MkCanvas $w.can $CN_STATS_WIDTH $CN_STATS_HEIGHT
	set stats_displayed true
	stats_created $w.can
    } elseif {[wm state $w] == "withdrawn"} {
	wm deiconify $w
	set stats_displayed true
    } else {
	wm withdraw $w
	set stats_displayed false
    }
}

proc MkCanvasFrame {w name cmd} {
    global CN_UPDATE_FREQ

    frame $w
    button $w.close \
	-text	 "Close $name" \
	-command $cmd \
	-relief raised -bd 2
    label $w.update -text $CN_UPDATE_FREQ

    pack $w.close $w.update -side left -fill x -expand yes
    pack $w -side top -fill x -expand no
}

# ----------------------------------------------------------------

proc ToggleNodeAttributes {n nodename nodetype mainwin x y \
			   width height_stats height_stdio} {

    global node_displayed trace_events
    global node_choice_title node_choice_label node_choice_set

    set w .node$n
    if {[info command $w] == ""} {

	if {$nodename == "default"} {
	    regsub -all "(\\+|\\x)" [wm geometry $mainwin] " " geom
	    set x [expr 100 + [lindex $geom 2]]
	    set y [expr  20 + [lindex $geom 1] + [lindex $geom 3]]
	}
	toplevel $w
	wm title $w $nodename
	wm geometry $w +$x+$y

	set f [frame $w.top]
	button $f.close \
	    -text	"Close $nodename node attributes" \
	    -command	"wm withdraw $w" \
	    -relief raised -bd 2
	checkbutton $f.trace -text	"trace all events" \
	    -variable "trace_events($n)" \
	    -relief flat

	pack $f.close $f.trace -side left -anchor e -expand no
	pack $w.top -side top -fill x -expand no

	if {$nodetype != "router"} {
	    set nn 0
	    if {$nodename == "default"} {
		set nn -1
	    }
	    foreach d {0 1 2} {
		set f [frame $w.choice$d]
		label $f.lab \
		    -width 20 \
		    -text	$node_choice_title($d)
		pack $f.lab -anchor w -side left
		foreach c {0 1 2 3} {
		    radiobutton $f.$c \
			-width  9 \
			-text   $node_choice_label($nn,$d,$c) \
			-value $c \
			-variable $f.selected \
			-relief raised -bd 2 \
			-command "node_choice $n $d $c"
		}
		$f.$node_choice_set($nn,$d) select
		pack $f.0 $f.1 $f.2 $f.3 -side left -fill x -expand yes
	    }
	    pack $w.choice0 $w.choice1 $w.choice2 -side top -fill x -expand no
	}
	if {$nodetype != "default"} {
	    if {$nodetype == "host"} {
		MkCanvas $w.stats $width $height_stats
	    }

	    set f [frame $w.d]
	    foreach i {0 1 2 3 4} {
		button "$f.debug$i"	-text " " \
		    -command	"node_debug_button $n $i" \
		    -width 8 \
		    -relief raised -bd 2
	    }
	    pack $f.debug0 $f.debug1 $f.debug2 $f.debug3 $f.debug4 \
		-side left -fill x -expand yes
	    pack $f -side top -padx 4 -pady 2 -fill x -expand no

	    MkCanvas $w.stdio $width $height_stdio

	    node_created $n $w $w.stats $w.stdio
	    bind $w.stdio <Enter> "focus $w.stdio"
	    bind $w.stdio <Key>   "stdio_input $n %N"
	}
	set node_displayed($n) true
    } elseif {[wm state $w] == "withdrawn"} {
	wm deiconify $w
	set node_displayed($n) true
    } else {
	wm withdraw $w
	set node_displayed($n) false
    }
}

proc SetDebugString {w str} {
    $w configure -text $str
}

# ----------------------------------------------------------------

proc ToggleLinkAttributes {from to linkname mainwin x y \
					stats_width stats_height} {
    global delay_title delay_label delay_set
    global link_scale_title link_scale_max link_scale_value
    global link_displayed

    set w .link($from,$to)
    if {[info command $w] == ""} {

	regsub -all "(\\+|\\x)" [wm geometry $mainwin] " " geom
	if {$linkname == "default"} {
	    set x [expr [lindex $geom 0] + [lindex $geom 2] - 100]
	    set y [expr [lindex $geom 1] + [lindex $geom 3] +  20]
	} else {
	    set x [expr $x + [lindex $geom 2] + 30]
	    set y [expr $y + [lindex $geom 3] + 90]
	}
	toplevel $w
	wm title $w "$linkname link"
	wm geometry $w +$x+$y

	set t [frame $w.top]
	button $t.close \
	    -text	"Close $linkname link attributes" \
	    -command	"wm withdraw $w" \
	    -relief raised -bd 2
	pack $t.close -side left -anchor e \
	    -expand no
	pack $t -side top -fill x -expand yes

	set min  0
	if {$linkname != "default"} {
	    MkCanvas $w.stats $stats_width $stats_height
	    set min -1
	}

	link_created $w $from $to $w.stats

	set b [frame $w.bottom]
	set f [frame $b.delay]
	label $f.lab -width 30 -text	$delay_title
	pack $f.lab -anchor w -side left

	foreach c {0 1 2 3} {
	    radiobutton $f.$c \
		-width  7 \
		-text   "$delay_label($c)" \
		-value $c \
		-variable $f.selected \
		-relief raised -bd 2 \
		-command "link_propagationdelay $from $to $c"
	}
	$f.$delay_set select
	pack $f.0 $f.1 $f.2 $f.3 -side left -fill x -expand yes

	foreach c {0 1 2 3} {
	    set f [frame $b.scale$c]
	    label $f.lab \
		-width 30 \
		-text	$link_scale_title($c)
	    pack $f.lab -anchor w -side left
	    scale $f.s	-from $min -to $link_scale_max($c) \
			-orient horizontal \
			-length 30 \
			-showvalue 1 \
			-command "link_scale $from $to $c"
	    $f.s set $link_scale_value($c)
	    pack $f.s -side left -fill x -expand yes
	}

	pack $b.delay $b.scale0 $b.scale1 $b.scale2 $b.scale3 \
	    -side top -fill x -expand no
	pack $b -side top -fill x -expand yes

	set link_displayed($from,$to) true

    } elseif {[wm state $w] == "withdrawn"} {
	wm deiconify $w
	set link_displayed($from,$to) true
    } else {
	wm withdraw $w
	set link_displayed($from,$to) false
    }
}

# ----------------------------------------------------------------

set node_menu_displayed -1
# only allow one nodemenu

proc PopupNodeMenu {n nodename mainwin activemask x y} {
    global node_menu_label node_menu_displayed tk_version

    set m .nodemenu
    if {[info command $m] == ""} {
	if {$tk_version > 3.6} \
	    then {menu $m -tearoff false} \
	    else {menu $m}

	$m add command -label $nodename -state disabled
	$m add separator
	foreach s {0 1 2 3 4} {
	    $m add command -label $node_menu_label($s) \
			   -command "PopdownNodeMenu $m $s"
	}
    }
    if {$node_menu_displayed != -1} {
	set node_menu_displayed -1
	$m unpost
	return
    }

    $m entryconfigure 0 -label $nodename
    foreach s {0 1 2 3 4} {
	$m entryconfigure [expr $s + 2] \
	    -state [expr {[expr $activemask&(1<<$s)] ? "normal" : "disabled"}]
    }
    set node_menu_displayed $n

    regsub -all "(\\+|\\x)" [wm geometry $mainwin] " " geom
    set x [expr $x + [lindex $geom 2]]
    set y [expr $y + [lindex $geom 3] + 10]
    $m post $x $y
}

proc PopdownNodeMenu {m s} {
    global node_menu_displayed node_menu_select

    node_menu_select $node_menu_displayed $s
    set node_menu_displayed -1
    $m unpost
}


set link_menu_displayed -1
# only allow one linkmenu

proc PopupLinkMenu {l linkname mainwin activemask x y} {
    global link_menu_label link_menu_displayed tk_version

    set m .linkmenu
    if {[info command $m] == ""} {
	if {$tk_version > 3.6} \
	    then {menu $m -tearoff false} \
	    else {menu $m}

	$m add command -label $linkname -state disabled
	$m add separator
	foreach s {0 1 2 3} {
	    $m add command -label $link_menu_label($s) \
			   -command "PopdownLinkMenu $m $s"
	}
    }
    if {$link_menu_displayed != -1} {
	set link_menu_displayed -1
	$m unpost
	return
    }

    $m entryconfigure 0 -label $linkname
    foreach s {0 1 2 3} {
	$m entryconfigure [expr $s + 2] \
	    -state [expr {[expr $activemask&(1<<$s)] ? "normal" : "disabled"}]
    }
    set link_menu_displayed $l

    regsub -all "(\\+|\\x)" [wm geometry $mainwin] " " geom
    set x [expr $x + [lindex $geom 2]]
    set y [expr $y + [lindex $geom 3] + 10]
    $m post $x $y
}

proc PopdownLinkMenu {m s} {
    global link_menu_displayed link_menu_select

    link_menu_select $link_menu_displayed $s
    set link_menu_displayed -1
    $m unpost
}
